unit MWconnU;

// Please, don't delete this comment. \\
(*
  Copyright Owner: coltishWARE (Kevin Niehage)
  Copyright Year : 2007

  Unit   : MWconnU (platform dependant)
  Version: 0.3c

  Contact E-Mail            : development@coltishWARE.com
  Contact Mobile            : +49-(0)-177-69-4321-8
  Contact Telephone Berlin  : +49-(0)-30-69-20-96-78
  Contact Telephone Cottbus : +49-(0)-355-28925-88-17-55
  Contact Telephone Mannheim: +49-(0)-621-483-4664
*)
// Please, don't delete this comment. \\

(*
  Description:

  This unit contains the shared memory interface
  implementation for the application MWconn by
  Markus B. Weber.

  It is a plain C to Pascal translation.
*)

(*
  Change Log:

  [Version 0.3c] (11.12.2007: 3.10 release)
  - renamed types and compiler switches

  [Version 0.3b] (16.11.2007: 3.9C release)
  - support for 3.9C (not just 3.9 stable)

  [Version 0.2c] (13.11.2007: extended release)
  - calling method split into three methods (shared memory accessible now)
  - command constants introduced

  [Version 0.1c] (13.11.2007: initial release)
  - structure definition translated from C to Pascal
  - calling method written
*)

interface

// define if latest version is to be used
{$DEFINE MWCONN_LATEST}

{$IFDEF MWCONN_LATEST}
  // should define the latest version available
  
  {$DEFINE MWCONN_3_10}
{$ELSE}
  // if MWCONN_LATEST is not defined the version to be used
  // has to be defined correctly

  {.$DEFINE MWCONN_3_10}
  {$DEFINE MWCONN_3_9}
{$ENDIF}

uses
  Windows,
  SysUtils;

const
  cCopyrightOwner = 'coltishWARE (Kevin Niehage)';
  cCopyrightYear  = '2007';
  cName           = 'MWconnU';
  cReleaseDate    = '11.12.2007';
  cReleaseName    = '3.10 release';
  cVersion        = '0.3c';

(* MWconn 3.10
typedef struct { // Struktur fr Datenaustausch mit externen Programmen
  unsigned long len; // Lnge dieser Struktur in Bytes

  unsigned short structure_version; // Version dieser Struktur: 2
  unsigned short program_version;   // Version des MWconn-Programms
                                    // (Hauptversion*256+Nebenversion)

  unsigned long res1[3]; // reserviert

  // Datenrichtung extern->MWconn
  char command[100]; // von MWconn auszufhrendes Kommando;
                     // "MWCONN_CONNECT": verbinden;
                     // "MWCONN_DISCONNECT": trennen;
                     // "MWCONN_RECONNECT": neu verbinden;
                     // "MWCONN_END": Programm beenden;
                     // ":::ATI": AT-Kommando an Gert senden (hier "ATI");
                     // bei Novatel-Gerten nur mit Novatel-DLL;
                     // ":250:4000:ATI": desgleichen, jedoch mit
                     // Wartezeitangaben, 250ms nach dem Senden
                     // beginnt der Empfang, welcher maximal 4 weitere
                     // Sekunden spter abgeschlossen sein muss;

  // Datenrichtung MWconn->extern
  // Hinweis: nicht alle Datenkarten untersttzen alle Informationen;
  unsigned long process_counter; // dieser Zhler wird von MWconn mit
                                 // jedem Verarbeitungstakt inkrementiert;
  unsigned long volume;          // summiertes Datenvolumen in kBytes
  unsigned long time;            // summierte Onlinezeit in Minuten
  unsigned long online_time;     // aktuelle Onlinezeit in Sekunden; ==0: offline
  unsigned long offline_time;    // aktuelle/letzte Offline-Zeit in Sekunden
  unsigned long upspeed;         // aktuelle Sendegeschwindigkeit in Bytes/s
  unsigned long downspeed;       // aktuelle Empfangsgeschwindigkeit in Bytes/s
  unsigned char opmode;          // Betriebsart; 0: nicht bekannt; 1: GPRS; 2: UMTS;
  unsigned char signal_raw;      // Signalstrke als GSM-Standardwert (0..31)
  unsigned char signal_percent;  // Signalstrke in Prozent (0..100);
                                 // nur gltig, wenn signal_raw>0
  signed short signal_dbm;       // Signalstrke in dBm
  char network_name[32+1];       // Name des Netzwerks
  char plmn[5+1];                // Public Land Mobile Network, bestehend aus MCC und MNC
  char lac[4+1];                 // Location Area Code
  char cid[4+1];                 // Cell-ID
  char message[64+1];            // ggf. Meldungstext
  char mini_message[48+1];       // ggf. verkrzter Meldungstext
  char error_message[32+1];      // ggf. Fehlermeldung

  unsigned char res2[3]; // reserviert
  unsigned long res3[7]; // reserviert

  char answer[600]; // Antwort auf AT-Kommando
} mwcio_t;
MWconn 3.10 *)

type
  mwcio_t_3_10 = record
    Len : LongWord;

    Structure_Version : Word;
    Program_Version   : Word;

    Res1 : array [0..2] of LongWord;

    Command : array [0..99] of Char;

    Process_Counter : LongWord;

    Volume         : LongWord;
    Time           : LongWord;
    Online_Time    : LongWord;
    Offline_Time   : LongWord;
    UpSpeed        : LongWord;
    DownSpeed      : LongWord;
    OpMode         : Byte; // unsigned char
    Signal_Raw     : Byte; // unsigned char
    Signal_Percent : Byte; // unsigned char
    Signal_DBM     : SmallInt;
    Network_Name   : array [0..32] of Char;
    PLMN           : array [0..5]  of Char;
    LAC            : array [0..4]  of Char;
    CID            : array [0..4]  of Char;
    Message        : array [0..64] of Char;
    Mini_Message   : array [0..48] of Char;
    Error_Message  : array [0..32] of Char;

    Res2 : array [0..2] of Byte; // unsigned char
    Res3 : array [0..6] of LongWord;

    Answer : array [0..599] of Char;
  end;

(* MWconn 3.9
typedef struct { // Struktur fr Datenaustausch mit externen Programmen
  unsigned long len; // Lnge dieser Struktur in Bytes

  unsigned short structure_version; // Version dieser Struktur: 1
  unsigned short program_version;   // Version des MWconn-Programms
                                    // (Hauptversion*256+Nebenversion)

  unsigned long res1[3]; // reserviert

  // Datenrichtung extern->MWconn
  char command[100]; // von MWconn auszufhrendes Kommando;
                     // "MWCONN_CONNECT": verbinden;
                     // "MWCONN_DISCONNECT": trennen;
                     // "MWCONN_RECONNECT": neu verbinden;
                     // "MWCONN_END": Programm beenden;

  // Datenrichtung MWconn->extern
  // Hinweis: nicht alle Datenkarten untersttzen alle Informationen;
  unsigned long process_counter; // dieser Zhler wird von MWconn mit
                                 // jedem Verarbeitungstakt inkrementiert;

  unsigned long volume;         // summiertes Datenvolumen in kBytes
  unsigned long time;           // summierte Onlinezeit in Minuten
  unsigned long online_time;    // aktuelle Onlinezeit in Sekunden; ==0: offline
  unsigned long offline_time;   // aktuelle Offline-Zeit in Sekunden
  unsigned long upspeed;        // aktuelle Sendegeschwindigkeit in Bytes/s
  unsigned long downspeed;      // aktuelle Empfangsgeschwindigkeit in Bytes/s
  unsigned char opmode;         // Betriebsart; 0: nicht bekannt; 1: GPRS; 2: UMTS;
  unsigned char signal_raw;     // Signalstrke als GSM-Standardwert (0..31)
  unsigned char signal_percent; // Signalstrke in Prozent (0..100);
                                // nur gltig, wenn signal_raw>0
  signed short signal_dbm;      // Signalstrke in dBm
  char network_name[32+1];      // Name des Netzwerks
  char plmn[5+1];               // Public Land Mobile Network, bestehend aus MCC und MNC
  char lac[4+1];                // Location Area Code
  char cid[4+1];                // Cell-ID
  char message[64+1];           // ggf. Meldungstext
  char mini_message[48+1];      // ggf. verkrzter Meldungstext
  char error_message[32+1];     // ggf. Fehlermeldung

  unsigned char res2[3];        // reserviert
  unsigned long res3[3];        // reserviert
} mwcio_t;
MWconn 3.9 *)

type
  mwcio_t_3_9 = record
    Len : LongWord;

    Structure_Version : Word;
    Program_Version   : Word;

    Res1 : array [0..2] of LongWord;

    Command : array [0..99] of Char;

    Process_Counter : LongWord;

    Volume         : LongWord;
    Time           : LongWord;
    Online_Time    : LongWord;
    Offline_Time   : LongWord;
    UpSpeed        : LongWord;
    DownSpeed      : LongWord;
    OpMode         : Byte; // unsigned char
    Signal_Raw     : Byte; // unsigned char
    Signal_Percent : Byte; // unsigned char
    Signal_DBM     : SmallInt;
    Network_Name   : array [0..32] of Char;
    PLMN           : array [0..5]  of Char;
    LAC            : array [0..4]  of Char;
    CID            : array [0..4]  of Char;
    Message        : array [0..64] of Char;
    Mini_Message   : array [0..48] of Char;
    Error_Message  : array [0..32] of Char;

    Res2 : array [0..2] of Byte; // unsigned char
    Res3 : array [0..2] of LongWord;
  end;

type
{$IFDEF MWCONN_3_10}
  TMWconnIO = mwcio_t_3_10;
{$ELSE}
  {$IFDEF MWCONN_3_9}
  TMWconnIO = mwcio_t_3_9;
  {$ELSE}
  // raise compiler error as no version is defined at all
  {$ENDIF}
{$ENDIF}

type
  PMWconnIO = ^TMWconnIO;

const
  MWconnFile = 'MWCONN_IO';

const
  MWconn_Connect    = 'MWCONN_CONNECT';
  MWconn_Disconnect = 'MWCONN_DISCONNECT';
  MWconn_Reconnect  = 'MWCONN_RECONNECT';
  MWconn_End        = 'MWCONN_END';

// opens the shared memory interface of MWconn
// use CloseMWconnIO() to close it again
function OpenMWconnIO(var aFile : THandle; var aMapping : PMWconnIO) : Boolean;

// closes an already open shared memory interface
// of MWconn opened with OpenMWconnIO()
procedure CloseMWconnIO(var aFile : THandle; var aMapping : PMWconnIO);

// returns a data snapshot returned by MWconn
function ReturnMWconnIO(var aData : TMWconnIO) : Boolean;

// duration values less than 0 are replaced by empty strings
// durations are millisecond values
// the result is: ":<aWaitDuration>:<aReadDuration>:<aCommand>"
function BuildATCommand(const aCommand : String; const aWaitDuration : LongInt; const aReadDuration : LongInt) : String;

implementation

// opens the shared memory interface of MWconn
// use CloseMWconnIO() to close it again
function OpenMWconnIO(var aFile : THandle; var aMapping : PMWconnIO) : Boolean;
begin
  Result := false;

  aFile := OpenFileMapping(FILE_MAP_ALL_ACCESS, false, MWconnFile);
  if (aFile <> 0) then
  begin
    try
      aMapping := MapViewOfFile(aFile, FILE_MAP_ALL_ACCESS, 0, 0, SizeOf(TMWconnIO));

      Result := (aMapping <> nil);
    except
      CloseHandle(aFile);
    end;
  end;
end;

// closes an already open shared memory interface
// of MWconn opened with OpenMWconnIO()
procedure CloseMWconnIO(var aFile : THandle; var aMapping : PMWconnIO);
begin
  if (aMapping <> nil) then
    UnmapViewOfFile(aMapping);
  if (aFile <> 0) then
    CloseHandle(aFile);

  aFile    := 0;
  aMapping := nil;
end;

// returns a data snapshot returned by MWconn
function ReturnMWconnIO(var aData : TMWconnIO) : Boolean;
var
  lBuffer : PMWconnIO;
  lHandle : THandle;
begin
  Result := false;

  if OpenMWconnIO(lHandle, lBuffer) then
  begin
    try
      CopyMemory(@aData, lBuffer, SizeOf(TMWconnIO));

      Result := true;
    finally
      CloseMWconnIO(lHandle, lBuffer);
    end;
  end;
end;

// duration values less than 0 are replaced by empty strings
// durations are millisecond values
// the result is: ":<aWaitDuration>:<aReadDuration>:<aCommand>"
function BuildATCommand(const aCommand : String; const aWaitDuration : LongInt; const aReadDuration : LongInt) : String;
const
  cSeperator = ':';
begin
  Result := cSeperator;

  if (aWaitDuration >= 0) then
    Result := Result + IntToStr(aWaitDuration);
  Result := Result + cSeperator;

  if (aReadDuration >= 0) then
    Result := Result + IntToStr(aReadDuration);
  Result := Result + cSeperator;

  Result := Result + aCommand;
end;

end.
